import { ComponentType } from 'react';
import Store from '../store/Store';
import hoistStatics from 'hoist-non-react-statics';
export declare type CompoundDependency<T> = {
    propTypes?: Object;
    stores: Store<any>[];
    deref: (props?: Object, state?: Object, stores?: Store<any>[]) => T;
};
export declare type Dependency<T> = CompoundDependency<T> | Store<T>;
export declare type DependencyIndexEntry = {
    dispatchTokens: {
        [key: string]: boolean;
    };
    fields: {
        [key: string]: boolean;
    };
};
export declare type DependencyIndex = {
    [key: string]: DependencyIndexEntry;
};
export declare type DependencyMap = {
    readonly [key: string]: Dependency<any>;
};
export declare type GetProps<C> = C extends ComponentType<infer P> ? P extends {} ? P : never : never;
export declare type GetDependencyType<D> = D extends Dependency<infer T> ? T : never;
export declare type DependencyMapProps<DM extends DependencyMap> = DM extends DependencyMap ? {
    [key in keyof DM]: GetDependencyType<DM[key]>;
} : never;
export declare type ConnectedComponentType<P, C> = React.ForwardRefExoticComponent<P & C> & {
    WrappedComponent: C;
    dependencies: DependencyMap;
} & hoistStatics.NonReactStatics<P & C>;
export declare type PropTypes = {
    [key: string]: Function;
};
export declare function enforceValidDependencies(dependencies: DependencyMap): DependencyMap;
export declare function dependencyPropTypes(dependencies: DependencyMap, existingPropTypes?: {
    [key: string]: Function;
}): PropTypes;
export declare function calculate<Props, State, DerefValue>(dependency: Dependency<DerefValue>, props?: Props, state?: State): DerefValue;
export declare function calculateInitial<Props, State, Deps extends DependencyMap>(dependencies: Deps, props: Props, state?: State): {
    [key in keyof Deps]: GetDependencyType<Deps[key]>;
};
export declare function calculateForDispatch<Props, State, Deps extends DependencyMap>(dependencies: Deps, dependencyIndexEntry: DependencyIndexEntry, props: Props, state?: State): {
    [key in keyof Deps]: Deps[key];
};
export declare function makeDependencyIndex(dependencies: DependencyMap): DependencyIndex;
