"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useCurrent = useCurrent;
exports._useDispatchSubscription = _useDispatchSubscription;
exports.default = void 0;

var _react = require("react");

var _DependencyMap = require("./DependencyMap");

var _DispatcherInstance = require("../dispatcher/DispatcherInstance");

var _DispatcherInterface = require("../dispatcher/DispatcherInterface");

var _Dispatch = require("./Dispatch");

var _ObjectUtils = require("../utils/ObjectUtils");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function useCurrent(value) {
  var ref = (0, _react.useRef)(value);
  ref.current = value;
  return ref;
}

function _useDispatchSubscription(dependencyMap, currentProps, dispatcher, dependencyValue, setDependencyValue) {
  (0, _react.useEffect)(function () {
    var dependencyIndex = (0, _DependencyMap.makeDependencyIndex)(dependencyMap);
    var dispatchToken = dispatcher.register(_Dispatch.handleDispatch.bind(null, dispatcher, dependencyIndex, function (entry) {
      var newValue = (0, _DependencyMap.calculateForDispatch)(dependencyMap, entry, currentProps.current, dependencyValue);

      if (Object.keys(newValue).map(function (k) {
        return (0, _ObjectUtils.shallowEqual)(newValue[k], dependencyValue[k]);
      }).some(function (el) {
        return !el;
      })) {
        setDependencyValue(newValue);
      }
    }));
    return function () {
      dispatcher.unregister(dispatchToken);
    };
  }, [currentProps, dependencyMap, dependencyValue, dispatcher, setDependencyValue]);
}

function useStoreDependency(dependency, props) {
  var dispatcher = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : (0, _DispatcherInstance.get)();
  (0, _DispatcherInterface.enforceDispatcher)(dispatcher);

  var _useState = (0, _react.useState)({
    dependency: (0, _DependencyMap.calculate)(dependency, props)
  }),
      _useState2 = _slicedToArray(_useState, 2),
      dependencyValue = _useState2[0],
      setDependencyValue = _useState2[1];

  var currProps = useCurrent(props);
  var dependencyMap = (0, _react.useMemo)(function () {
    return {
      dependency: dependency
    };
  }, [dependency]);

  _useDispatchSubscription(dependencyMap, currProps, dispatcher, dependencyValue, setDependencyValue);

  var newValue = {
    dependency: (0, _DependencyMap.calculate)(dependency, props)
  };

  if (!(0, _ObjectUtils.shallowEqual)(newValue.dependency, dependencyValue.dependency)) {
    setDependencyValue(newValue);
  }

  return dependencyValue.dependency;
}

var _default = useStoreDependency;
exports.default = _default;