import EventHandler from './EventHandler';
export default class Event {
    _handlers: {
        [key: string]: Function;
    };
    constructor();
    /**
     * Add a subscription to this event
     *
     * @param  callback  run when the event is triggered.
     * @return this
     */
    addHandler(callback: Function): EventHandler;
    /**
     * Destroys this event. Removes all handlers.
     *
     * @return this
     */
    remove(): Event;
    /**
     * Removes a subscription by key.
     *
     * @param  key   id of the subscription to remove
     * @return this
     */
    removeHandler(key: string): Event;
    /**
     * @protected
     * Run a handler by key if it exists
     *
     * @param  key  id of the handler to run
     */
    _runHandler(key: string): void;
    /**
     * Run all subscribed handlers.
     *
     * @return this
     */
    runHandlers(): Event;
    /**
     * Convenience method for running multiple events.
     *
     * @param  events  a list of events to run.
     */
    static runMultiple(events: Array<Event>): void;
}
